/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 18/09/2014
**/
#include <qpainter.h>
#include <ctype.h>
#include <glu.h>
#include "pantalla.h"

Pantalla::Pantalla( QWidget *parent )
		: QGLWidget( parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer|QGL::NoDepthBuffer ), parent )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	// setFocusPolicy( QWidget::StrongFocus);
	setFocusPolicy( Qt::StrongFocus);

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );

	crea_cubo( cubo1, -0.6, -0.6, -0.6, 0.6, 0.6, 0.6 );
}


void Pantalla::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Pantalla::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Pantalla::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Pantalla::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	float a = 0.8;

	if( w > h )
		glOrtho (-a*aspect, a*aspect, -a, a, -a, a );
	else
		glOrtho ( -a, a, -a/aspect, a/aspect, -a, a );
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	emit changeSize ( );
}

QSizePolicy Pantalla::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Pantalla::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	glRotatef ( xrot , 1.0, 0.0, 0.0);
	glRotatef ( yrot , 0.0, 1.0, 0.0);

	dibuja_cubo( cubo1 );

	swapBuffers( );
}

void Pantalla::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0 );

	glEnable (GL_DEPTH_TEST);

}


void Pantalla::crea_cubo( double c[][3], double x1, double y1, double z1,
                          double x8, double y8, double z8 )
{
	c[0][0] = x1; c[0][1] = y1; c[0][2] = z1; 
	c[1][0] = x8; c[1][1] = y1; c[1][2] = z1; 
	c[2][0] = x8; c[2][1] = y1; c[2][2] = z8; 
	c[3][0] = x1; c[3][1] = y1; c[3][2] = z8; 

	c[4][0] = x8; c[4][1] = y8; c[4][2] = z1; 
	c[5][0] = x1; c[5][1] = y8; c[5][2] = z1; 
	c[6][0] = x1; c[6][1] = y8; c[6][2] = z8; 
	c[7][0] = x8; c[7][1] = y8; c[7][2] = z8; 
}


void Pantalla::dibuja_cubo( double c[][3] )
{
	int i;
	int caras[6][4] = { { 2, 7, 6, 3 },
                        { 4, 5, 6, 7 },
                        { 1, 4, 7, 2 },
                        { 0, 5, 4, 1 },
                        { 0, 1, 2, 3 },
                        { 0, 5, 6, 3 } };

	float colores[6][3] = { { 1.0, 1.0, 1.0 },   // blanco -> frontal 
							{ 1.0, 0.0, 0.0 },   // rojo   -> superior
							{ 0.0, 1.0, 0.0 },   // verde  -> lateral derecha
							{ 0.0, 0.0, 1.0 },   // azul   -> trasera
							{ 0.0, 1.0, 1.0 },   // amarillo -> inferior
							{ 1.0, 1.0, 0.0 }, }; // cian  -> lateral izquierda


	glBegin( GL_QUADS );

	for( i=0; i<6; i++ ) {
		glColor3fv( &colores[i][0] );

		glVertex3dv( &c[ caras[i][0] ][0] );
		glVertex3dv( &c[ caras[i][1] ][0] );
		glVertex3dv( &c[ caras[i][2] ][0] );
		glVertex3dv( &c[ caras[i][3] ][0] );
	}

	glEnd( );
}

